# frozen_string_literal: true
require 'fileutils'

class S3StorageService
  # Reference: https://docs.aws.amazon.com/sdk-for-ruby/v3/developer-guide/
  # https://docs.ceph.com/en/latest/radosgw/s3/ruby/
  # https://docs.aws.amazon.com/sdk-for-ruby/v3/developer-guide/s3-example-create-buckets.html
  attr_reader :s3_client

  def init_client
    @s3_client = Aws::S3::Client.new
  end

  def sanitise_name(bucket_name)
    if ENV.fetch('S3_BUCKET_PREFIX', '')
      "#{ENV['S3_BUCKET_PREFIX']}-#{bucket_name}".downcase.gsub(/[^0-9a-z.-]/, '')
    else
      bucket_name.downcase.gsub(/[^0-9a-z.-]/, '')
    end
  end

  def list_buckets
    buckets = []
    @s3_client.list_buckets.buckets.each do |bucket|
      buckets << {
        name: bucket.name,
        creation_date: bucket.creation_date,
        size: bucket.size
      }
    end
    buckets
  end

  def bucket_exists?(bucket_name)
    @s3_client.list_buckets.buckets.each do |bucket|
      return true if bucket.name == bucket_name
    end
    false
  end

  def list_objects(bucket_name, prefix = nil, max_keys = 1000)
    # To get the first max_keys objects in a bucket
    contents = []
    total_size = 0
    objects = @s3_client.list_objects_v2(
      bucket: bucket_name,
      max_keys: max_keys,
      prefix: prefix
    ).contents
    objects.each do |object|
      contents << {
        key: object.key,
        size: object.size,
        last_modified_date: object.last_modified
      }
      total_size += object.size
    end
    return contents, total_size
  end

  def list_all_objects(bucket_name, prefix = nil)
    contents = []
    total_size = 0
    resp = @s3_client.list_objects_v2(
      bucket: bucket_name,
      max_keys: 1000,
      prefix: prefix
    )
    loop do
      objects = resp.contents
      objects.each do |object|
        contents << {
          key: object.key,
          size: object.size,
          last_modified_date: object.last_modified
        }
        total_size += object.size
      end
      break unless resp.next_page?

      resp = resp.next_page
    end
    return contents, total_size
  end

  def get_content(bucket_name, object_key, local_file_path)
    @s3_client.get_object(
      bucket: bucket_name,
      key: object_key,
      response_target: local_file_path
    )
  end

  def create_bucket(bucket_name)
    response = @s3_client.create_bucket(bucket: bucket_name)
    return true if response.location == "/#{bucket_name}"

    false
  end

  def add_content(bucket_name, object_key, object_content)
    response = s3_client.put_object(
      bucket: bucket_name,
      key: object_key,
      body: object_content
      # content_type: 'text/plain'
    )
    return true if response.etag

    false
  end

  def add_multipart_content(_bucket_name, _object_key, _file_path)
    # ToDo
    false
  end

  def delete_object(bucket_name, object_key)
    response = @s3_client.delete_objects(
      bucket: bucket_name,
      delete: {
        objects: [
          {
            key: object_key
          }
        ]
      }
    )
    return true if response.deleted.count == 1

    false
  end

  def delete_objects(bucket_name, object_keys)
    delete_keys = []
    object_keys.each do |object_key|
      delete_keys << { key: object_key }
    end
    response = @s3_client.delete_objects(
      bucket: bucket_name,
      delete: {
        objects: object_keys
      }
    )
    return true if response.deleted.count == object_keys.count

    false
  end

  def delete_all_objects(bucket_name)
    Aws::S3::Bucket.new(bucket_name, client: @s3_client).clear!
  end

  def delete_bucket(bucket_name)
    response = @s3_client.delete_bucket(bucket: bucket_name)
  end

  def move_object(source_object_key, target_bucket_name, target_object_key, metadata = {})
    file_upload_bucket = sanitise_name(ENV['S3_FILE_UPLOAD_BUCKET'])
    source_bucket = Aws::S3::Resource
                             .new(region: ENV['S3_REGION'])
                             .bucket(file_upload_bucket)

    source_object = source_bucket.object(source_object_key)

    source_object.move_to(bucket: target_bucket_name, key: target_object_key, metadata: metadata)
  rescue Aws::Errors::ServiceError => e
    puts "Couldn't copy #{source_object.key} to #{target_object_key}. Here's why: #{e.message}"
  end

  def download_bucket(bucket)
    s3_bucket = sanitise_name(bucket)
    return false unless bucket_exists?(s3_bucket)
    list_of_objects, total_size = list_all_objects(s3_bucket)
    return false if total_size > ENV.fetch('DOWNLOAD_SIZE_LIMIT', '1000000000').to_i
    base_dir = File.join(ENV.fetch('DOWNLOAD_PATH', 'tmp/downloads'), bucket)
    list_of_objects.each do |object|
      path = File.join(base_dir, object[:key])
      dirname = File.dirname(path)
      FileUtils.mkdir_p(dirname) unless File.directory?(dirname)
      self.get_content(s3_bucket, object[:key], path)
    end
    base_dir
  end
end
